from typing import Final

PAGE_TITLE: Final = "insta-ui"
SCOPED_STYLE_GROUP_ID: Final = "insta-scoped-style"
DEFAULT_SLOT_NAME: Final = "default"
MOUNTED_EVENT_NAME: Final = "mounted"
STORAGE_REF_METHOD: Final = "storageRef"
INSTAUI_JS_MODULE_NAME: Final = "instaui"
USE_DARK_REF_METHOD: Final = "useDarkRef"
USE_LANGUAGE_METHOD: Final = "useLanguageRef"
USE_PAGE_TITLE_METHOD: Final = "usePageTitleRef"
USE_ROUTE_PARAM_METHOD: Final = "useRouteParam"
USE_QUERY_PARAM_METHOD: Final = "useQueryParam"

USER_ASSETS_DIR_NAME: Final = "assets"
USER_ASSETS_ICONS_DIR_NAME: Final = "icons"
